<?php
/**
 * A filter for checking if value "encrypt" is set and fine
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2014 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class CaptchaEncryptFilter extends BaseFilter implements Filterable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this filter class
	 *
	 * @return	$filterInstance			An instance of this filter class
	 */
	public static final function createCaptchaEncryptFilter () {
		// Get a new instance
		$filterInstance = new CaptchaEncryptFilter();

		// Return the instance
		return $filterInstance;
	}

	/**
	 * Executes the filter with given request and response objects
	 *
	 * @param	$requestInstance	An instance of a class with an Requestable interface
	 * @param	$responseInstance	An instance of a class with an Responseable interface
	 * @return	void
	 * @throws	EncryptMissingException		If the "encrypt" value is not set
	 */
	public function execute (Requestable $requestInstance, Responseable $responseInstance) {
		// Get "encrypt" string barely from the request
		$encryptRequest = $requestInstance->getRequestElement('encrypt');

		// Is it there?
		if (is_null($encryptRequest)) {
			// Not found, so request is invalid
			$requestInstance->requestIsValid(FALSE);

			// Throw exception
			throw new EncryptMissingException($this, CryptoHelper::EXCEPTION_ENCRYPT_MISSING);
		} // END - if

		// Decode it fully
		$encryptDecoded = base64_decode(str_replace(' ', '+', urldecode($encryptRequest)));

		// Get a crypto helper and decrypt the string
		$decryptedString = ObjectFactory::createObjectByConfiguredName('crypto_class')->decryptString($encryptDecoded);

		// Is it the expected length?
		if (strlen($decryptedString) != $this->getConfigInstance()->getConfigEntry('captcha_string_length')) {
			// Not found, so request is invalid
			$requestInstance->requestIsValid(FALSE);

			// Throw exception
			throw new EncryptInvalidLengthException($this, CryptoHelper::EXCEPTION_ENCRYPT_INVALID);
		} // END - if

		// Write it to the request
		$requestInstance->setRequestElement('decrypted', $decryptedString);
	}
}

// [EOF]
?>
